@@ -81,7 +81,6 @@ class RubyLexer
   # grammar use.
 
   attr_accessor :lex_state
-
   attr_accessor :lex_strterm
   attr_accessor :lpar_beg
   attr_accessor :paren_nest
@@ -120,10 +119,6 @@ class RubyLexer
   end
   alias :bol? :beginning_of_line? # to make .rex file more readable
 
-  def check re
-    ss.check re
-  end
-
   def comments # TODO: remove this... maybe comment_string + attr_accessor
     c = @comments.join
     @comments.clear
@@ -278,10 +273,18 @@ class RubyLexer
     in_lex_state? :expr_end, :expr_endarg, :expr_endfn
   end
 
+  def ruby22_label?
+    ruby22? and is_label_possible?
+  end
+
   def is_label_possible?
     (in_lex_state?(:expr_beg, :expr_endfn) && !command_state) || is_arg?
   end
 
+  def is_label_suffix?
+    check(/:(?!:)/)
+  end
+
   def is_space_arg? c = "x"
     is_arg? and space_seen and c !~ /\s/
   end
@@ -335,6 +338,12 @@ class RubyLexer
     when "}" then
       self.brace_nest -= 1
       self.lex_state   = :expr_endarg
+
+      # TODO
+      # if (c == '}') {
+      #     if (!brace_nest--) c = tSTRING_DEND;
+      # }
+
       return :tRCURLY, matched
     when "]" then
       self.paren_nest -= 1
@@ -449,7 +458,7 @@ class RubyLexer
     self.lineno += matched.lines.to_a.size if scan(/\n+/)
 
     return if in_lex_state?(:expr_beg, :expr_value, :expr_class,
-                            :expr_fname, :expr_dot)
+                            :expr_fname, :expr_dot, :expr_labelarg)
 
     if scan(/([\ \t\r\f\v]*)\./) then
       self.space_seen = true unless ss[1].empty?
@@ -653,6 +662,29 @@ class RubyLexer
     return result(:expr_end, :tSYMBOL, symbol)
   end
 
+  def was_label?
+    @was_label = ruby22_label?
+    true
+  end
+
+  def process_label_or_string text
+    if @was_label && text =~ /:\Z/ then
+      @was_label = nil
+      return process_label text
+    elsif text =~ /:\Z/ then
+      ss.pos -= 1 # put back ":"
+      text = text[0..-2]
+    end
+
+    result :expr_end, :tSTRING, text[1..-2].gsub(/\\\\/, "\\").gsub(/\\'/, "'")
+  end
+
+  def process_label text
+    symbol = text[1..-3].gsub(ESC) { unescape $1 }
+
+    result(:expr_labelarg, :tLABEL, [symbol, self.lineno])
+  end
+
   def process_token text
     # TODO: make this always return [token, lineno]
     token = self.token = text
@@ -673,7 +705,8 @@ class RubyLexer
         :tIDENTIFIER
       end
 
-    if !ruby18 and is_label_possible? and scan(/:(?!:)/) then
+    if !ruby18 and is_label_possible? and is_label_suffix? then
+      scan(/:/)
       return result(:expr_labelarg, :tLABEL, [token, self.lineno])
     end
 
@@ -734,7 +767,7 @@ class RubyLexer
       else
         result(state, :kDO, value)
       end
-    when in_lex_state?(:expr_beg, :expr_value) then
+    when in_lex_state?(:expr_beg, :expr_value) then # TODO: :expr_labelarg
       result(state, keyword.id0, value)
     when keyword.id0 != keyword.id1 then
       result(:expr_beg, keyword.id1, value)
@@ -866,6 +899,10 @@ class RubyLexer
     ss.scan re
   end
 
+  def check re
+    ss.check re
+  end
+
   def scanner_class # TODO: design this out of oedipus_lex. or something.
     RPStringScanner
   end
@@ -1036,6 +1073,11 @@ class RubyLexer
     # do nothing for now
   end
 
+  def ruby22?
+    Ruby22Parser === parser or
+      Ruby23Parser === parser
+  end
+
   def process_string # TODO: rewrite / remove
     token = if lex_strterm[0] == :heredoc then
               self.heredoc lex_strterm
@@ -1043,11 +1085,20 @@ class RubyLexer
               self.parse_string lex_strterm
             end
 
-    token_type, _ = token
+    token_type, c = token
 
-    if token_type == :tSTRING_END || token_type == :tREGEXP_END then
+    if ruby22? && token_type == :tSTRING_END && ["'", '"'].include?(c) then
+      if (([:expr_beg, :expr_endfn].include?(lex_state) &&
+           !cond.is_in_state) || is_arg?) &&
+          is_label_suffix? then
+        scan(/:/)
+        token_type = token[0] = :tLABEL_END
+      end
+    end
+
+    if [:tSTRING_END, :tREGEXP_END, :tLABEL_END].include? token_type then
       self.lex_strterm = nil
-      self.lex_state   = :expr_end
+      self.lex_state   = (token_type == :tLABEL_END) ? :expr_labelarg : :expr_end
     end
 
     return token
@@ -1169,7 +1220,7 @@ end
 
 require "ruby_lexer.rex"
 
-if ENV["DEBUG"] then
+if ENV["RP_LINENO_DEBUG"] then
   class RubyLexer
     alias :old_lineno= :lineno=
 
