@@ -91,7 +91,7 @@ class RPStringScanner < StringScanner
 end
 
 module RubyParserStuff
-  VERSION = "3.6.6" unless constants.include? "VERSION" # SIGH
+  VERSION = "3.8.2" unless constants.include? "VERSION" # SIGH
 
   attr_accessor :lexer, :in_def, :in_single, :file
   attr_reader :env, :comments
@@ -557,8 +557,15 @@ module RubyParserStuff
     end
   end
 
-  def new_call recv, meth, args = nil
-    result = s(:call, recv, meth)
+  def new_call recv, meth, args = nil, call_op = :'.'
+    result = case call_op.to_sym
+             when :'.'
+               s(:call, recv, meth)
+             when :'&.'
+               s(:safe_call, recv, meth)
+             else
+               raise "unknown call operator: `#{type.inspect}`"
+             end
 
     # TODO: need a test with f(&b) to produce block_pass
     # TODO: need a test with f(&b) { } to produce warning
@@ -577,6 +584,22 @@ module RubyParserStuff
     result
   end
 
+  def new_attrasgn recv, meth, call_op
+    meth = :"#{meth}="
+
+    result = case call_op.to_sym
+             when :'.'
+               s(:attrasgn, recv, meth)
+             when :'&.'
+               s(:safe_attrasgn, recv, meth)
+             else
+               raise "unknown call operator: `#{type.inspect}`"
+             end
+
+    result.line = recv.line
+    result
+  end
+
   def new_case expr, body, line
     result = s(:case, expr)
 
@@ -645,6 +668,7 @@ module RubyParserStuff
 
   def new_defs val
     recv, name, args, body = val[1], val[4], val[6], val[7]
+    body ||= s(:nil)
 
     result = s(:defs, recv, name.to_sym, args)
 
@@ -744,6 +768,23 @@ module RubyParserStuff
     result
   end
 
+  def new_op_asgn2 val
+    recv, call_op, meth, op, arg = val
+    meth = :"#{meth}="
+
+    result = case call_op.to_sym
+             when :'.'
+               s(:op_asgn2, recv, meth, op.to_sym, arg)
+             when :'&.'
+               s(:safe_op_asgn2, recv, meth, op.to_sym, arg)
+             else
+               raise "unknown call operator: `#{type.inspect}`"
+             end
+
+    result.line = recv.line
+    result
+  end
+
   def new_regexp val
     node = val[1] || s(:str, '')
     options = val[2]
@@ -931,7 +972,7 @@ module RubyParserStuff
     rhs = value_expr rhs
 
     case lhs[0]
-    when :lasgn, :iasgn, :cdecl, :cvdecl, :gasgn, :cvasgn, :attrasgn then
+    when :lasgn, :iasgn, :cdecl, :cvdecl, :gasgn, :cvasgn, :attrasgn, :safe_attrasgn then
       lhs << rhs
     when :const then
       lhs[0] = :cdecl
@@ -1319,6 +1360,14 @@ module RubyParserStuff
   end
 end
 
+class Ruby23Parser < Racc::Parser
+  include RubyParserStuff
+end
+
+class Ruby22Parser < Racc::Parser
+  include RubyParserStuff
+end
+
 class Ruby21Parser < Racc::Parser
   include RubyParserStuff
 end
@@ -1348,11 +1397,13 @@ class RubyParser
     @p19 = Ruby19Parser.new
     @p20 = Ruby20Parser.new
     @p21 = Ruby21Parser.new
+    @p22 = Ruby22Parser.new
+    @p23 = Ruby23Parser.new
   end
 
   def process s, f = "(string)", t = 10
     e = nil
-    [@p21, @p20, @p19, @p18].each do |parser|
+    [@p23, @p22, @p21, @p20, @p19, @p18].each do |parser|
       begin
         return parser.process s, f, t
       rescue Racc::ParseError, RubyParser::SyntaxError => exc
@@ -1369,6 +1420,8 @@ class RubyParser
     @p19.reset
     @p20.reset
     @p21.reset
+    @p22.reset
+    @p23.reset
   end
 
   def self.for_current_ruby
@@ -1381,6 +1434,10 @@ class RubyParser
       Ruby20Parser.new
     when /^2.1/ then
       Ruby21Parser.new
+    when /^2.2/ then
+      Ruby22Parser.new
+    when /^2.3/ then
+      Ruby23Parser.new
     else
       raise "unrecognized RUBY_VERSION #{RUBY_VERSION}"
     end
