@@ -86,7 +86,12 @@ class TestRubyLexer < Minitest::Test
     act_value = act_value.first if Array === act_value
 
     assert_equal token, act_token,       msg
-    assert_equal value, act_value,       msg
+    case value
+    when Float then
+      assert_in_epsilon value, act_value, 0.001, msg
+    else
+      assert_equal value, act_value,       msg
+    end
     assert_equal state, @lex.lex_state,  msg if state
     assert_equal paren, @lex.paren_nest, msg if paren
     assert_equal brace, @lex.brace_nest, msg if brace
@@ -240,6 +245,21 @@ class TestRubyLexer < Minitest::Test
     assert_lex3("&&=", nil, :tOP_ASGN, "&&", :expr_beg)
   end
 
+  def test_yylex_and_dot
+    setup_lexer_class Ruby23Parser
+
+    assert_lex3("&.", nil, :tLONELY, "&.", :expr_dot)
+  end
+
+  def test_yylex_and_dot_call
+    setup_lexer_class Ruby23Parser
+
+    assert_lex3("x&.y", nil,
+                :tIDENTIFIER, "x", :expr_cmdarg,
+                :tLONELY, "&.", :expr_dot,
+                :tIDENTIFIER, "y")
+  end
+
   def test_yylex_and_arg
     self.lex_state = :expr_arg
 
@@ -518,10 +538,10 @@ class TestRubyLexer < Minitest::Test
   def test_yylex_const_call_same_name
     assert_lex("X = a { }; b { f :c }",
                s(:block,
-                 s(:cdecl, :X, s(:iter, s(:call, nil, :a), s(:args))),
+                 s(:cdecl, :X, s(:iter, s(:call, nil, :a), 0)),
                  s(:iter,
                    s(:call, nil, :b),
-                   s(:args),
+                   0,
                    s(:call, nil, :f, s(:lit, :c)))),
 
                :tCONSTANT,   "X", :expr_cmdarg, 0, 0,
@@ -539,10 +559,10 @@ class TestRubyLexer < Minitest::Test
 
     assert_lex("X = a { }; b { X :c }",
                s(:block,
-                 s(:cdecl, :X, s(:iter, s(:call, nil, :a), s(:args))),
+                 s(:cdecl, :X, s(:iter, s(:call, nil, :a), 0)),
                  s(:iter,
                    s(:call, nil, :b),
-                   s(:args),
+                   0,
                    s(:call, nil, :X, s(:lit, :c)))),
 
                :tCONSTANT,   "X", :expr_cmdarg, 0, 0,
@@ -2294,10 +2314,12 @@ class TestRubyLexer < Minitest::Test
   end
 
   def test_yylex_string_double_pound_dollar_bad
+    skip if Ruby18Parser === lexer.parser
+
     assert_lex3('"#$%"', nil,
 
                 :tSTRING_BEG,     "\"",  :expr_beg,
-                :tSTRING_CONTENT, "#$%", :expr_beg,
+                :tSTRING_CONTENT, '#$%', :expr_beg,
                 :tSTRING_END,     "\"",  :expr_end)
   end
 
@@ -2641,15 +2663,39 @@ class TestRubyLexer < Minitest::Test
   end
 
   def test_yylex_hash_colon_quoted_22
-    skip "Waiting for 2.2 parser"
+    setup_lexer_class Ruby22Parser
 
     assert_lex("{'a':1}",
                s(:hash, s(:lit, :a), s(:lit, 1)),
 
-               :tLBRACE, "{", :expr_beg,    0, 1,
-               :tSYMBOL, "a", :expr_end,    0, 1,
-               :tINTEGER, 1,  :expr_end,    0, 1,
-               :tRCURLY, "}", :expr_endarg, 0, 0)
+               :tLBRACE, "{", :expr_beg,      0, 1,
+               :tLABEL,  "a", :expr_labelarg, 0, 1,
+               :tINTEGER, 1,  :expr_end,      0, 1,
+               :tRCURLY, "}", :expr_endarg,   0, 0)
+  end
+
+  def test_yylex_hash_colon_quoted_symbol_22
+    setup_lexer_class Ruby22Parser
+
+    assert_lex("{'abc': :b}",
+               s(:hash, s(:lit, :abc), s(:lit, :b)),
+
+               :tLBRACE, "{", :expr_beg,      0, 1,
+               :tLABEL,  "abc", :expr_labelarg, 0, 1,
+               :tSYMBOL, "b",  :expr_end,     0, 1,
+               :tRCURLY, "}", :expr_endarg,   0, 0)
+  end
+
+  def test_yylex_hash_colon_double_quoted_symbol_22
+    setup_lexer_class Ruby22Parser
+
+    assert_lex('{"abc": :b}',
+               s(:hash, s(:lit, :abc), s(:lit, :b)),
+
+               :tLBRACE, "{", :expr_beg,      0, 1,
+               :tLABEL,  "abc", :expr_labelarg, 0, 1,
+               :tSYMBOL, "b",  :expr_end,     0, 1,
+               :tRCURLY, "}", :expr_endarg,   0, 0)
   end
 
   def test_ruby21_new_numbers
