@@ -121,7 +121,7 @@ module TestRubyParserShared
     rb = "a.b (1) {c}"
     pt = s(:iter,
            s(:call, s(:call, nil, :a), :b, s(:lit, 1)),
-           s(:args),
+           0,
            s(:call, nil, :c))
 
     assert_parse rb, pt
@@ -131,7 +131,7 @@ module TestRubyParserShared
     rb = "a::b (1) {c}"
     pt =  s(:iter,
             s(:call, s(:call, nil, :a), :b, s(:lit, 1)),
-            s(:args),
+            0,
             s(:call, nil, :c))
 
     assert_parse rb, pt
@@ -226,7 +226,7 @@ module TestRubyParserShared
     rb = "a do\n  v = nil\n  begin\n    yield\n  rescue Exception => v\n    break\n  end\nend"
     pt = s(:iter,
            s(:call, nil, :a),
-           s(:args),
+           0,
            s(:block,
              s(:lasgn, :v, s(:nil)),
              s(:rescue,
@@ -269,7 +269,7 @@ module TestRubyParserShared
 
   def test_defs_comments
     rb = "# blah 1\n# blah 2\n\ndef self.blah\nend"
-    pt = s(:defs, s(:self), :blah, s(:args))
+    pt = s(:defs, s(:self), :blah, s(:args), s(:nil))
 
     assert_parse rb, pt
     assert_equal "# blah 1\n# blah 2\n\n", result.comments
@@ -575,6 +575,21 @@ module TestRubyParserShared
     # TODO: add more including interpolation etc
   end
 
+  def test_label_vs_string
+    rb = "_buf << ':\n'"
+    pt = s(:call, s(:call, nil, :_buf), :<<, s(:str, ":\n"))
+
+    assert_parse rb, pt
+  end
+
+  def test_str_backslashes
+    long_string = '\n' * 100
+    rb = "x '#{long_string}'"
+    pt = s(:call, nil, :x, s(:str, long_string))
+
+    assert_parse rb, pt
+  end
+
   def test_str_pct_Q_nested
     rb = "%Q[before [#\{nest}] after]"
     pt = s(:dstr, "before [", s(:evstr, s(:call, nil, :nest)), s(:str, "] after"))
@@ -942,6 +957,14 @@ module TestRubyParserShared
     Ruby21Parser === self.processor
   end
 
+  def ruby22
+    Ruby22Parser === self.processor
+  end
+
+  def ruby23
+    Ruby23Parser === self.processor
+  end
+
   def test_bug_comma
     val = if ruby18 then
             s(:lit, 100)
@@ -997,7 +1020,7 @@ module TestRubyParserShared
     rb = "not(a)"
     pt = if ruby18 then
            s(:not, s(:call, nil, :a))
-         elsif ruby19 or ruby20 or ruby21 then
+         elsif ruby19 or ruby20 or ruby21 or ruby22 or ruby23 then
            s(:call, s(:call, nil, :a), :"!")
          else
            raise "wtf"
@@ -1008,7 +1031,7 @@ module TestRubyParserShared
 
   def test_pipe_space
     rb = "a.b do | | end"
-    pt = s(:iter, s(:call, s(:call, nil, :a), :b), 0)
+    pt = s(:iter, s(:call, s(:call, nil, :a), :b), s(:args))
 
     assert_parse rb, pt
   end
@@ -1311,7 +1334,7 @@ module TestRubyParserShared
 
   def test_defns_reserved
     rb = "def self.return; end"
-    pt = s(:defs, s(:self), :return, s(:args))
+    pt = s(:defs, s(:self), :return, s(:args), s(:nil))
 
     assert_parse rb, pt
   end
@@ -1378,6 +1401,19 @@ module TestRubyParserShared
     assert_parse rb, pt
   end
 
+  def test_qwords_line_breaks
+    skip "not yet"
+
+    rb = "%w(\na\nb\n)\n1"
+    pt = s(:block,
+           s(:array,
+             s(:str, "a").line(2),
+             s(:str, "b").line(3)).line(1),
+           s(:lit, 1).line(5))
+
+    assert_parse rb, pt
+  end
+
   def test_qWords_space
     rb = "%W( )"
     pt = s(:array)
@@ -1591,7 +1627,7 @@ module TestRubyParserShared
   end
 end
 
-module TestRubyParserShared19to21
+module TestRubyParserShared19to22
   def test_aref_args_lit_assocs
     rb = "[1, 2 => 3]"
     pt = s(:array, s(:lit, 1), s(:hash, s(:lit, 2), s(:lit, 3)))
@@ -1610,7 +1646,7 @@ module TestRubyParserShared19to21
     rb = "a.b c do end.d"
     pt = s(:call,
            s(:iter,
-             s(:call, s(:call, nil, :a), :b, s(:call, nil, :c)), s(:args)),
+             s(:call, s(:call, nil, :a), :b, s(:call, nil, :c)), 0),
            :d)
 
     assert_parse rb, pt
@@ -1620,7 +1656,7 @@ module TestRubyParserShared19to21
     rb = "a.b c do end::d"
     pt = s(:call,
            s(:iter,
-             s(:call, s(:call, nil, :a), :b, s(:call, nil, :c)), s(:args)),
+             s(:call, s(:call, nil, :a), :b, s(:call, nil, :c)), 0),
            :d)
 
     assert_parse rb, pt
@@ -1629,7 +1665,7 @@ module TestRubyParserShared19to21
   def test_block_command_operation_dot
     rb = "a :b do end.c :d"
     pt = s(:call,
-           s(:iter, s(:call, nil, :a, s(:lit, :b)), s(:args)),
+           s(:iter, s(:call, nil, :a, s(:lit, :b)), 0),
            :c,
            s(:lit, :d))
 
@@ -1639,7 +1675,7 @@ module TestRubyParserShared19to21
   def test_block_command_operation_colon
     rb = "a :b do end::c :d"
     pt = s(:call,
-           s(:iter, s(:call, nil, :a, s(:lit, :b)), s(:args)),
+           s(:iter, s(:call, nil, :a, s(:lit, :b)), 0),
            :c,
            s(:lit, :d))
 
@@ -1707,6 +1743,17 @@ module TestRubyParserShared19to21
     assert_parse rb, pt
   end
 
+  def test_bug_187
+    rb = "private def f\na.b do end\nend"
+    pt = s(:call,
+           nil,
+           :private,
+           s(:defn, :f, s(:args),
+             s(:iter, s(:call, s(:call, nil, :a), :b), 0)))
+
+    assert_parse rb, pt
+  end
+
   def test_defn_opt_reg
     rb = "def f(a=nil, b) end"
     pt = s(:defn, :f, s(:args, s(:lasgn, :a, s(:nil)), :b), s(:nil))
@@ -2006,7 +2053,7 @@ module TestRubyParserShared19to21
 
   def test_do_lambda
     rb = "->() do end"
-    pt = s(:iter, s(:call, nil, :lambda), 0)
+    pt = s(:iter, s(:call, nil, :lambda), s(:args))
 
     assert_parse rb, pt
   end
@@ -2129,9 +2176,24 @@ module TestRubyParserShared19to21
 
     assert_parse rb, pt
   end
+
+  def test_multiline_hash_declaration
+    pt = s(:call, nil, :f, s(:hash, s(:lit, :state), s(:hash)))
+
+    assert_parse "f(state: {})",     pt
+    assert_parse "f(state: {\n})",   pt
+    assert_parse "f(state:\n {\n})", pt
+  end
 end
 
-module TestRubyParserShared20to21
+module TestRubyParserShared20to22
+  def test_defs_kwarg
+    rb = "def self.a b: 1\nend"
+    pt = s(:defs, s(:self), :a, s(:args, s(:kwarg, :b, s(:lit, 1))), s(:nil))
+
+    assert_parse rb, pt
+  end
+
   def test_defn_kwarg_kwsplat
     rb = "def a(b: 1, **c) end"
     pt = s(:defn, :a, s(:args, s(:kwarg, :b, s(:lit, 1)), :"**c"), s(:nil))
@@ -2147,6 +2209,13 @@ module TestRubyParserShared20to21
     assert_parse rb, pt
   end
 
+  def test_dstr_lex_state
+    rb = '"#{p:a}"'
+    pt = s(:dstr, "", s(:evstr, s(:call, nil, :p, s(:lit, :a))))
+
+    assert_parse rb, pt
+  end
+
   def test_call_arg_kwsplat
     rb = "a(b, **1)"
     pt = s(:call, nil, :a, s(:call, nil, :b), s(:hash, s(:kwsplat, s(:lit, 1))))
@@ -2442,7 +2511,7 @@ end
 
 class TestRuby19Parser < RubyParserTestCase
   include TestRubyParserShared
-  include TestRubyParserShared19to21
+  include TestRubyParserShared19to22
 
   def setup
     super
@@ -2681,7 +2750,7 @@ class TestRuby19Parser < RubyParserTestCase
            s(:lit, :a),
            s(:iter,
              s(:call, nil, :lambda),
-             s(:args),
+             0,
              s(:if, s(:call, nil, :b), s(:call, nil, :c), s(:call, nil, :d))),
 
            s(:lit, :e),
@@ -2690,12 +2759,14 @@ class TestRuby19Parser < RubyParserTestCase
     assert_parse rb, pt
   end
 
-  # def test_pipe_semicolon # HACK
-  #   rb = "a.b do | ; c | end"
-  #   pt = s(:iter, s(:call, s(:call, nil, :a), :b), 0)
-  #
-  #   assert_parse rb, pt
-  # end
+  def test_pipe_semicolon
+    skip "not yet"
+
+    rb = "a.b do | ; c | end"
+    pt = s(:iter, s(:call, s(:call, nil, :a), :b), 0)
+
+    assert_parse rb, pt
+  end
 
   def test_wtf
     # lambda -> f_larglist lambda_body
@@ -2799,12 +2870,14 @@ class TestRuby19Parser < RubyParserTestCase
     assert_parse rb, pt
   end
 
-  # def test_kill_me5
-  #   rb = "f ->() { g do end }"
-  #   pt = 42
-  #
-  #   assert_parse rb, pt
-  # end
+  def test_kill_me5
+    skip "not yet"
+
+    rb = "f ->() { g do end }"
+    pt = 42
+
+    assert_parse rb, pt
+  end
 
   def test_iter_args_4
     rb = "f { |a, *b, c| }"
@@ -2814,6 +2887,8 @@ class TestRuby19Parser < RubyParserTestCase
   end
 
   def test_iter_args_5
+    skip "not yet"
+
     rb = "f { |a, &b| }"
     pt = s(:iter, s(:call, nil, :f), s(:args, :a, :"&b"))
 
@@ -3013,13 +3088,21 @@ class TestRuby19Parser < RubyParserTestCase
   end
 
   def test_lambda_do_vs_brace
-    pt = s(:call, nil, :f, s(:iter, s(:call, nil, :lambda), 0))
+    pt = s(:call, nil, :f, s(:iter, s(:call, nil, :lambda), s(:args)))
 
     rb = "f ->() {}"
     assert_parse rb, pt
 
     rb = "f ->() do end"
     assert_parse rb, pt
+
+    pt = s(:call, nil, :f, s(:iter, s(:call, nil, :lambda), 0))
+
+    rb = "f -> {}"
+    assert_parse rb, pt
+
+    rb = "f -> do end"
+    assert_parse rb, pt
   end
 
   def test_thingy
@@ -3042,8 +3125,8 @@ end
 
 class TestRuby20Parser < RubyParserTestCase
   include TestRubyParserShared
-  include TestRubyParserShared20to21
-  include TestRubyParserShared19to21
+  include TestRubyParserShared20to22
+  include TestRubyParserShared19to22
 
   def setup
     super
@@ -3057,7 +3140,7 @@ class TestRuby20Parser < RubyParserTestCase
            s(:call,
              s(:iter,
                s(:call, s(:call, nil, :a), :b, s(:call, nil, :c)),
-               s(:args),
+               0,
                s(:call, nil, :d)),
              :e),
            s(:args, :f),
@@ -3073,7 +3156,7 @@ class TestRuby20Parser < RubyParserTestCase
            s(:call,
              s(:iter,
                s(:call, s(:call, nil, :a), :b, s(:call, nil, :c)),
-               s(:args),
+               0,
                s(:call, nil, :d)),
              :e,
              s(:call, nil, :f)),
@@ -3213,8 +3296,8 @@ end
 
 class TestRuby21Parser < RubyParserTestCase
   include TestRubyParserShared
-  include TestRubyParserShared19to21
-  include TestRubyParserShared20to21
+  include TestRubyParserShared19to22
+  include TestRubyParserShared20to22
 
   def setup
     super
@@ -3289,3 +3372,172 @@ class TestRuby21Parser < RubyParserTestCase
     assert_parse rb, pt
   end
 end
+
+class TestRuby22Parser < RubyParserTestCase
+  include TestRubyParserShared
+  include TestRubyParserShared19to22
+  include TestRubyParserShared20to22
+
+  def setup
+    super
+
+    self.processor = Ruby22Parser.new
+  end
+
+  def test_call_args_assoc_quoted
+    pt = s(:call, nil, :x, s(:hash, s(:lit, :k), s(:lit, 42)))
+
+    rb = "x 'k':42"
+    assert_parse rb, pt
+
+    rb = 'x "k":42'
+    assert_parse rb, pt
+
+    rb = 'x "#{k}":42'
+    pt = s(:call, nil, :x, s(:hash, s(:dsym, "", s(:evstr, s(:call, nil, :k))), s(:lit, 42)))
+
+    assert_parse rb, pt
+  end
+
+  def test_bug191
+    pt = s(:if, s(:call, nil, :a), s(:str, ""), s(:call, nil, :b))
+
+    rb = "a ? '': b"
+    assert_parse rb, pt
+
+    rb = "a ? \"\": b"
+    assert_parse rb, pt
+  end
+
+  def test_quoted_symbol_keys
+    rb = "{ 'a': :b }"
+    pt = s(:hash, s(:lit, :a), s(:lit, :b))
+
+    assert_parse rb, pt
+  end
+
+  def test_quoted_symbol_hash_arg
+    rb = "puts 'a': {}"
+    pt = s(:call, nil, :puts, s(:hash, s(:lit, :a), s(:hash)))
+
+    assert_parse rb, pt
+  end
+end
+
+class TestRuby23Parser < RubyParserTestCase
+  include TestRubyParserShared
+  include TestRubyParserShared19to22
+  include TestRubyParserShared20to22
+
+  def setup
+    super
+
+    self.processor = Ruby23Parser.new
+  end
+
+  def test_safe_call
+    rb = "a&.b"
+    pt = s(:safe_call, s(:call, nil, :a), :b)
+
+    assert_parse rb, pt
+  end
+
+  def test_safe_call_newline
+    rb = "a&.b\n"
+    pt = s(:safe_call, s(:call, nil, :a), :b)
+
+    assert_parse rb, pt
+  end
+
+  def test_safe_call_rhs_newline
+    rb = "c = a&.b\n"
+    pt = s(:lasgn, :c, s(:safe_call, s(:call, nil, :a), :b))
+
+    assert_parse rb, pt
+  end
+
+  def test_safe_calls
+    rb = "a&.b&.c(1)"
+    pt = s(:safe_call, s(:safe_call, s(:call, nil, :a), :b), :c, s(:lit, 1))
+
+    assert_parse rb, pt
+  end
+
+  def test_safe_attrasgn
+    rb = "a&.b = 1"
+    pt = s(:safe_attrasgn, s(:call, nil, :a), :"b=", s(:lit, 1))
+
+    assert_parse rb, pt
+  end
+
+  def test_safe_attrasgn_constant
+    rb = "a&.B = 1"
+    pt = s(:safe_attrasgn, s(:call, nil, :a), :"B=", s(:lit, 1))
+
+    assert_parse rb, pt
+  end
+
+  def test_safe_call_dot_parens
+    rb = "a&.()"
+    pt = s(:safe_call, s(:call, nil, :a), :call)
+
+    assert_parse rb, pt
+  end
+
+  def test_safe_call_operator
+    rb = "a&.> 1"
+    pt = s(:safe_call, s(:call, nil, :a), :>, s(:lit, 1)).line(1)
+
+    assert_parse rb, pt
+  end
+
+  def test_safe_op_asgn
+    rb = "a&.b += x 1\n"
+    pt = s(:safe_op_asgn, s(:call, nil, :a), s(:call, nil, :x, s(:lit, 1)), :b, :+).line(1)
+
+    assert_parse rb, pt
+  end
+
+  def test_safe_op_asgn2
+    rb = "a&.b ||=\nx;"
+    pt = s(:safe_op_asgn2, s(:call, nil, :a), :b=, :"||", s(:call, nil, :x)).line(1)
+
+    assert_parse rb, pt
+  end
+end
+
+
+[18, 19, 20, 21, 22, 23].each do |v|
+  describe "block args arity #{v}" do
+    attr_accessor :parser
+
+    before do
+      self.parser = Object.const_get("Ruby#{v}Parser").new
+    end
+
+    {
+     "->       {    }" => s(:iter, s(:call, nil, :lambda),           0),
+     "lambda   {    }" => s(:iter, s(:call, nil, :lambda),           0),
+     "proc     {    }" => s(:iter, s(:call, nil, :proc),             0),
+     "Proc.new {    }" => s(:iter, s(:call, s(:const, :Proc), :new), 0),
+
+     "-> ()    {    }" => s(:iter, s(:call, nil, :lambda),           s(:args)),
+     "lambda   { || }" => s(:iter, s(:call, nil, :lambda),           s(:args)),
+     "proc     { || }" => s(:iter, s(:call, nil, :proc),             s(:args)),
+     "Proc.new { || }" => s(:iter, s(:call, s(:const, :Proc), :new), s(:args)),
+
+    }.each do |input, expected|
+      next if v == 18 and input =~ /->/
+      next if v == 19 and input =~ /-> \(\)/
+
+      it "parses '#{input}'" do
+        assert_equal expected, parser.parse(input)
+      end
+
+      input = input.sub(/\{/, "do").sub(/\}/, "end")
+      it "parses '#{input}'" do
+        assert_equal expected, parser.parse(input)
+      end
+    end
+  end
+end
